@@ -21,7 +21,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: DSIGSignature.hpp,v 1.26 2005/06/04 11:30:25 blautenb Exp $
+ * $Id: DSIGSignature.hpp 351395 2005-07-23 02:11:54Z blautenb $
  *
  */
 
@@ -278,6 +278,34 @@ public:
 
 	bool getPrettyPrint(void);
 
+	/**
+	 * \brief Create a \<Signature\> DOM structure.
+	 *
+	 * <p>The DOM structure created is still divorced from the document.  The callee
+	 * needs to take the returned \<Signature\> Element node and insert it at the
+	 * appropriate place in their document.</p>
+	 *
+	 * <p>The signature is a skeleton only.  There are no references or KeyInfo
+	 * elements inserted.  However the DSIGSignature structures are set up with the
+	 * new information, so once an element has been created and a signing key has been
+	 * set, a call to #sign will sign appropriately.
+	 *
+	 * @note The digest method (hash method) set here is for the signing function only.
+	 * Different hash methods can be used for reference elements.
+	 *
+	 * @param doc The document the Signature DOM structure will be inserted into.
+	 * @param canonicalizationAlgorithmURI The canonicalisation method to use.
+	 * @param signatureAlgorithmURI The Signature algorithm (defined by URI) to use
+	 * @returns The newly created \<Signature\> element that the caller should insert in
+	 * the document.
+	 */
+
+	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *createBlankSignature(
+		XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc,
+		const XMLCh * canonicalizationAlgorithmURI,
+		const XMLCh * signatureAlgorithmURI
+	);
+
 	/**
 	 * \brief Create a \<Signature\> DOM structure.
 	 *
@@ -299,6 +327,8 @@ public:
 	 * @param hm The Digest function to be used for the actual signatures.
 	 * @returns The newly created \<Signature\> element that the caller should insert in
 	 * the document.
+	 * @deprecated Use the URI based creation method instead, as this supports 
+	 * signature algorithms that are not known to the library directly.
 	 */
 
 	XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *createBlankSignature(
@@ -324,6 +354,7 @@ public:
 	 * @param hm The hashing (digest) method to be used for this reference
 	 * @param type A "type" string (as defined in XML Signature).
 	 * @returns The newly created DSIGReference element.
+	 * @deprecated Use the URI based creation method instead
 	 *
 	 */
 
@@ -334,6 +365,31 @@ public:
 		char * type = NULL
 	);
 
+	/**
+	 * \brief Add a new reference to the end of the list of \<Reference\> nodes.
+	 *
+	 * <p>Creates a new DSIGReference, adds it to the list of references handled
+	 * by the owning DSIGSignature and also creates the skeleton DOM structure into
+	 * the document.</p>
+	 *
+	 * @note The XSEC Library currently makes very little use of <em>type</em>
+	 * attributes in \<Reference\> Elements.  However this may of use to calling
+	 * applications.
+	 *
+	 * @see DSIGReference
+	 * @param URI The Data that this Reference node refers to.
+	 * @param hm The hashing (digest) method to be used for this reference
+	 * @param type A "type" string (as defined in XML Signature).
+	 * @returns The newly created DSIGReference element.
+	 * @deprecated Use the URI based creation method instead
+	 *
+	 */
+
+	DSIGReference * createReference(
+		const XMLCh * URI,
+		const XMLCh * hashAlgorithmURI, 
+		const XMLCh * type = NULL
+	);
 	//@}
 
 	/** @name General and Information functions. */
@@ -513,6 +569,49 @@ public:
 
 	XSECSafeBufferFormatter * getSBFormatter(void) {return mp_formatter;}
 
+	/**
+	 * \brief Set the interlocking references flag
+	 *
+	 * This flag controls whether the library will cater for "interlocking"
+	 * Reference nodes - where the hash of one reference depends on the
+	 * hash of another.
+	 *
+	 * As of Version 1.3, this is, by default, set to false, as it is 
+	 * fairly unusual to find a Reference node that has as an input the
+	 * digest value of another reference node.  By setting this value to
+	 * true, the library will first calculate all hash values, and then
+	 * verify the hash values.  If any are incorrect (caused by one 
+	 * the value of one reference being changed *after* it was used as 
+	 * the input to another), then the hash values are recalculated.  This
+	 * is repeated until the reference list "settles".
+	 *
+	 * @param flag true for interlocking references, false for standalone
+	 */
+
+	void setInterlockingReferences(bool flag) {m_interlockingReferences = flag;}
+
+
+	/**
+	 * \brief Get the interlocking references flag
+	 *
+	 * This flag controls whether the library will cater for "interlocking"
+	 * Reference nodes - where the hash of one reference depends on the
+	 * hash of another.
+	 *
+	 * As of Version 1.3, this is, by default, set to false, as it is 
+	 * fairly unusual to find a Reference node that has as an input the
+	 * digest value of another reference node.  By setting this value to
+	 * true, the library will first calculate all hash values, and then
+	 * verify the hash values.  If any are incorrect (caused by one 
+	 * the value of one reference being changed *after* it was used as 
+	 * the input to another), then the hash values are recalculated.  This
+	 * is repeated until the reference list "settles".
+	 *
+	 * @return Value of flag - true for interlocking references, false for standalone
+	 */
+
+	bool getInterlockingReferences(void) {return m_interlockingReferences;}
+
 	//@}
 
 	/** @name Resolver manipulation */
@@ -826,6 +925,10 @@ private:
 	// Internal functions
 	void createKeyInfoElement(void);
 	bool verifySignatureOnlyInternal(void);
+	TXFMChain * getSignedInfoInput(void);
+
+	// Initialisation
+	static void Initialise(void);
 
 	XSECSafeBufferFormatter		* mp_formatter;
 	bool						m_loaded;				// Have we already loaded?
@@ -855,9 +958,14 @@ private:
 
 	ObjectVectorType			m_objects;
 
+	// Interlocking references
+	bool						m_interlockingReferences;
+
 	// Not implemented constructors
 
 	DSIGSignature();
+	
+	friend class XSECPlatformUtils;
 
 	/*\@}*/
 };
