@@ -22,7 +22,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: OpenSSLCryptoSymmetricKey.cpp,v 1.15 2005/02/03 13:13:08 milan Exp $
+ * $Id: OpenSSLCryptoSymmetricKey.cpp 395833 2006-04-21 10:21:57Z blautenb $
  *
  */
 
@@ -440,7 +440,7 @@ unsigned int OpenSSLCryptoSymmetricKey::decryptFinish(unsigned char * plainBuf,
 
 	outl += offset;
 
-	if (m_doPad && offset > 0) {
+	if (m_doPad && outl > 0) {
 
 		/* Strip any padding */
 		outl -= scrPlainBuf[outl - 1];
