@@ -21,7 +21,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: WinCAPICryptoKeyRSA.cpp,v 1.11 2005/05/23 09:33:18 milan Exp $
+ * $Id: WinCAPICryptoKeyRSA.cpp 450743 2006-09-28 07:51:31Z blautenb $
  *
  */
 
@@ -31,6 +31,8 @@
 #include <xsec/enc/XSECCryptoException.hpp>
 #include <xsec/framework/XSECError.hpp>
 
+#if defined (HAVE_WINCAPI)
+
 #include <xercesc/util/Janitor.hpp>
 
 XSEC_USING_XERCES(ArrayJanitor);
@@ -60,7 +62,7 @@ WinCAPICryptoKeyRSA::~WinCAPICryptoKeyRSA() {
 
 	// If we have a RSA, delete it
 
-	if (m_key == 0)
+	if (m_key != 0)
 		CryptDestroyKey(m_key);
 
 	if (mp_exponent)
@@ -255,7 +257,8 @@ HCRYPTKEY WinCAPICryptoKeyRSA::importKey(void) {
 bool WinCAPICryptoKeyRSA::verifySHA1PKCS1Base64Signature(const unsigned char * hashBuf, 
 								 unsigned int hashLen,
 								 const char * base64Signature,
-								 unsigned int sigLen) {
+								 unsigned int sigLen,
+								 hashMethod hm) {
 
 	// Use the currently loaded key to validate the Base64 encoded signature
 
@@ -270,6 +273,20 @@ bool WinCAPICryptoKeyRSA::verifySHA1PKCS1Base64Signature(const unsigned char * h
 		}
 	}
 
+	/* Is this a hash we support? */
+	ALG_ID alg;
+	switch (hm) {
+	case (HASH_MD5):
+		alg = CALG_MD5;
+		break;
+	case (HASH_SHA1):
+		alg=CALG_SHA1;
+		break;
+	default:
+		throw XSECCryptoException(XSECCryptoException::RSAError,
+			"WinCAPI:RSA Unsupported hash algorithm for RSA sign - only MD5 or SHA1 supported");
+	}
+
 	// Decode the signature
 	unsigned char * rawSig;
 	DWORD rawSigLen;
@@ -299,7 +316,7 @@ bool WinCAPICryptoKeyRSA::verifySHA1PKCS1Base64Signature(const unsigned char * h
 	BOOL fResult;
 	HCRYPTHASH h;
 	fResult = CryptCreateHash(m_p, 
-					CALG_SHA1, 
+					alg, 
 					0, 
 					0,
 					&h);
@@ -335,13 +352,19 @@ bool WinCAPICryptoKeyRSA::verifySHA1PKCS1Base64Signature(const unsigned char * h
 		DWORD error = GetLastError();
 
 		if (error != NTE_BAD_SIGNATURE) {
+			if (h)
+				CryptDestroyHash(h);
 			throw XSECCryptoException(XSECCryptoException::RSAError,
 			"WinCAPI:RSA - Error occurred in RSA validation");
 		}
 
+		if (h)
+			CryptDestroyHash(h);
 		return false;
 	}
 
+	if (h)
+		CryptDestroyHash(h);
 	return true;
 
 }
@@ -354,7 +377,8 @@ bool WinCAPICryptoKeyRSA::verifySHA1PKCS1Base64Signature(const unsigned char * h
 unsigned int WinCAPICryptoKeyRSA::signSHA1PKCS1Base64Signature(unsigned char * hashBuf,
 		unsigned int hashLen,
 		char * base64SignatureBuf,
-		unsigned int base64SignatureBufLen) {
+		unsigned int base64SignatureBufLen,
+		hashMethod hm) {
 
 	// Sign a pre-calculated hash using this key
 
@@ -364,11 +388,25 @@ unsigned int WinCAPICryptoKeyRSA::signSHA1PKCS1Base64Signature(unsigned char * h
 			"WinCAPI:RSA - Attempt to sign data using a public or un-loaded key");
 	}
 
+	/* Is this a hash we support? */
+	ALG_ID alg;
+	switch (hm) {
+	case (HASH_MD5):
+		alg = CALG_MD5;
+		break;
+	case (HASH_SHA1):
+		alg=CALG_SHA1;
+		break;
+	default:
+		throw XSECCryptoException(XSECCryptoException::RSAError,
+			"WinCAPI:RSA Unsupported hash algorithm for RSA sign - only MD5 or SHA1 supported");
+	}
+
 	// Have to create a Windows hash object and feed in the hash
 	BOOL fResult;
 	HCRYPTHASH h;
 	fResult = CryptCreateHash(m_p, 
-					CALG_SHA1, 
+					alg, 
 					0, 
 					0,
 					&h);
@@ -386,6 +424,8 @@ unsigned int WinCAPICryptoKeyRSA::signSHA1PKCS1Base64Signature(unsigned char * h
 					0);
 
 	if (!fResult) {
+		if (h)
+			CryptDestroyHash(h);
 		throw XSECCryptoException(XSECCryptoException::RSAError,
 			"WinCAPI:RSA - Error Setting Hash Value in Windows Hash object");
 	}
@@ -402,6 +442,8 @@ unsigned int WinCAPICryptoKeyRSA::signSHA1PKCS1Base64Signature(unsigned char * h
 
 	if (!fResult || rawSigLen < 1) {
 
+		if (h)
+			CryptDestroyHash(h);
 		throw XSECCryptoException(XSECCryptoException::RSAError,
 		"WinCAPI:RSA - Error occurred obtaining RSA sig length");
 	}
@@ -420,10 +462,18 @@ unsigned int WinCAPICryptoKeyRSA::signSHA1PKCS1Base64Signature(unsigned char * h
 
 	if (!fResult || rawSigLen < 1) {
 
+		// Free the hash
+		if (h)
+			CryptDestroyHash(h);
+
 		throw XSECCryptoException(XSECCryptoException::RSAError,
 		"WinCAPI:RSA - Error occurred signing hash");
 	}
 
+	// Free the hash
+	if (h)
+		CryptDestroyHash(h);
+
 	// Now encode into a signature block
 	BYTE *rawSigFinal;
 	XSECnew(rawSigFinal, BYTE[rawSigLen]);
@@ -797,3 +847,5 @@ unsigned int WinCAPICryptoKeyRSA::getModulusBase64BigNums(char * b64, unsigned i
 	return bLen;
 
 }
+
+#endif /* WINCAPI */
