@@ -20,7 +20,7 @@
  * XSECCryptoSymmetricKey := Bulk encryption algorithms should all be
  *							implemented via this interface
  *
- * $Id: WinCAPICryptoSymmetricKey.cpp,v 1.11 2005/05/15 08:51:17 milan Exp $
+ * $Id: WinCAPICryptoSymmetricKey.cpp 394388 2006-04-15 22:42:13Z blautenb $
  *
  */
 
@@ -254,6 +254,17 @@ unsigned int WinCAPICryptoSymmetricKey::decrypt(const unsigned char * inBuf,
 		}
 	}
 
+	/* As per bug #38365 - we need to ensure there are enough characters to decrypt.
+	   Otherwise we get some nasty errors due to rounding calculations when inputs
+	   are too small */
+
+	else if (m_bytesInLastBlock + inLength < m_blockSize) {
+		// Not enough input data
+		memcpy(&m_lastBlock[m_bytesInLastBlock], inBuf, inLength);
+		m_bytesInLastBlock += inLength;
+		return 0;
+	}
+
 	DWORD outl = inLength - offset;
 
 	// Copy in last block
@@ -489,6 +500,19 @@ unsigned int WinCAPICryptoSymmetricKey::encrypt(const unsigned char * inBuf,
 
 	}
 
+	/* As per bug #38365 - we need to ensure there are enough characters to encrypt.
+	   Otherwise we get some nasty errors due to rounding calculations when inputs
+	   are too small */
+
+	if (m_bytesInLastBlock + inLength < m_blockSize) {
+		// Not enough input data
+		memcpy(&m_lastBlock[m_bytesInLastBlock], inBuf, inLength);
+		m_bytesInLastBlock += inLength;
+
+		// Just in case we have returned an IV
+		return outl;
+	}
+
 	bufPtr = &cipherBuf[offset];
 	encPtr = bufPtr;
 
@@ -682,7 +706,7 @@ HCRYPTKEY WinCAPICryptoSymmetricKey::createWindowsKey(
 	// Check key length - otherwise the user could get some very cryptic error messages
 	if (keyLen != expectedLength) {
 		throw XSECCryptoException(XSECCryptoException::SymmetricError,
-			"WinCAPI:SymmetricKey::createWindowsKey - Keylength incorrect for algorithm");
+			"WinCAPI:SymmetricKey::createWindowsKey - Key length incorrect for algorithm");
 	}
 
 	DWORD * algId = (DWORD *) (simpleBlob + sizeof(BLOBHEADER));
